import { ChangeDetectionStrategy, ViewEncapsulation } from './directive-config';
/**
 * Provides utility functions for generating Angular 2+ application code.
 */
var AngularWriter = /** @class */ (function () {
    function AngularWriter() {
    }
    /**
    * Writes a full Angular '@Component' class decorator using the provided configuration.
    */
    AngularWriter.writeComponentDecorator = function (writer, config) {
        writer.writeDecoratorCodeBlock('Component', function () {
            AngularWriter.writeComponentConfig(writer, config);
        });
    };
    /**
     * Writes Angular component configuration that can be used inside a component class decorator.
     * Use writeComponentDecorator() to write a full '@Component' class decorator.
     */
    AngularWriter.writeComponentConfig = function (writer, config) {
        var keys = Object.keys(config);
        keys.forEach(function (key, index) {
            writer.writeIndent();
            var value = config[key];
            writer.write(key + ": " + AngularWriter.stringifyComponentProperty(key, value));
            if (index < keys.length - 1)
                writer.write(',');
            writer.writeEndOfLine();
        });
    };
    /**
    * Writes a full Angular '@NgModule' class decorator using the provided configuration.
    */
    AngularWriter.writeModuleDecorator = function (writer, config) {
        writer.writeDecoratorCodeBlock('NgModule', function () {
            AngularWriter.writeModuleConfig(writer, config);
        });
    };
    /**
     * Writes Angular module configuration that can be used inside a module class decorator.
     * Use writeModuleDecorator() to write a full '@NgModule' class decorator.
     */
    AngularWriter.writeModuleConfig = function (writer, config) {
        var keys = Object.keys(config);
        keys.forEach(function (key, index) {
            writer.writeIndent();
            var value = config[key];
            writer.write(key + ": " + AngularWriter.stringifyModuleProperty(key, value));
            if (index < keys.length - 1)
                writer.write(',');
            writer.writeEndOfLine();
        });
    };
    /**
     * Writes Angular application configuration.
     */
    AngularWriter.writeApplicationConfig = function (writer, config) {
        var keys = Object.keys(config);
        keys.forEach(function (key, index) {
            writer.writeIndent();
            var value = config[key];
            writer.write(key + ": " + AngularWriter.stringifyAppConfigProperty(key, value));
            if (index < keys.length - 1)
                writer.write(',');
            writer.writeEndOfLine();
        });
    };
    /**
    * Writes a route configuration for the specified component, with the specified route path.
    */
    AngularWriter.writeRoute = function (writer, route) {
        writer.writeLine('{');
        writer.increaseIndent();
        writer.writeLine("path: '" + route.path + "',");
        writer.writeLine("component: " + route.componentName);
        writer.decreaseIndent();
        writer.writeLine('},');
    };
    AngularWriter.stringifyAppConfigProperty = function (key, value) {
        switch (key) {
            case 'providers': // done as multiline
                return AngularWriter.stringifyObjectArray(value);
            default:
                break;
        }
        if (typeof (value) == typeof (true)) {
            return value ? 'true' : 'false';
        }
        if (value instanceof Array) {
            return AngularWriter.stringifyStringArray(value);
        }
        // The value is a string
        else
            return "'" + value + "'";
    };
    AngularWriter.stringifyModuleProperty = function (key, value) {
        switch (key) {
            case 'bootstrap':
            case 'declarations':
            case 'entryComponents':
            case 'imports':
            case 'exports':
            case 'providers':
            case 'schemas':
                return AngularWriter.stringifyObjectArray(value);
            default: break;
        }
        if (typeof (value) == typeof (true)) {
            return value ? 'true' : 'false';
        }
        return "'" + value + "'";
    };
    AngularWriter.stringifyComponentProperty = function (key, value) {
        switch (key) {
            case 'changeDetection':
                return "ChangeDetectionStrategy." + ChangeDetectionStrategy[value];
            case 'encapsulation':
                return "ViewEncapsulation." + ViewEncapsulation[value];
            case 'entryComponents':
            case 'imports':
            case 'schemas':
            case 'providers':
            case 'viewProviders':
                return AngularWriter.stringifyObjectArray(value);
            case 'host':
                return AngularWriter.stringifyKeyValuePair(value);
            default:
                break;
        }
        if (typeof (value) == typeof (true)) {
            return value ? 'true' : 'false';
        }
        if (value instanceof Array) {
            return AngularWriter.stringifyStringArray(value);
        }
        // The value is a string
        else
            return "'" + value + "'";
    };
    AngularWriter.stringifyObjectArray = function (arr) {
        if (!arr.length)
            return '[]';
        return "[" + arr.join(', ') + "]";
    };
    AngularWriter.stringifyStringArray = function (arr) {
        if (!arr.length)
            return '[]';
        return "[" + arr.map(function (item) { return "'" + item + "'"; }).join(', ') + "]";
    };
    AngularWriter.stringifyKeyValuePair = function (value) {
        // {'key1': 'value1', 'key2': 'value2'}
        var keys = Object.keys(value);
        return "{ " + keys.map(function (k) { return "'" + k + "': '" + value[k] + "'"; }).join(', ') + " }";
    };
    return AngularWriter;
}());
export { AngularWriter };
